# dfx-bootstrap-table
An Angular Table CDK implementation for Bootstrap. Compatible with
[@ng-bootstrap/ng-bootstrap](https://npmjs.org/package/@ng-bootstrap/ng-bootstrap).

[![NPM](https://nodei.co/npm/dfx-bootstrap-table.png)](https://npmjs.org/package/dfx-bootstrap-table)

### Description
The ngb-table provides a Bootstrap Design styled data-table that can be used to display rows of data.

This table builds on the foundation of the CDK data-table and uses a similar interface for its data input and template, except that its element and attribute selectors will be prefixed with ngb- instead of cdk-. For more information on the interface and a detailed look at how the table is implemented, see the
[guide covering the CDK data-table](https://material.angular.io/guide/cdk-table).

### Information
- [Usage](#usage)
- [Development notes](#development)
- [Deployment notes](#deployment-notes)
- [Project website](https://datepoll.org)
- created with [Angular](https://angular.io),
  [used libaries](https://gitlab.com/DatePoll/common/dfx-bootstrap-table/-/blob/master/package.json)

### Credits
Full credits go to the [Angular and Angular Material](https://github.com/angular/components) Team. I literally copied most of their mat-table implementation and narrowed it down for Bootstrap.

## Usage
### Installation
```bash
npm install dfx-bootstrap-table@latest
```
* *If you have not already installed [Bootstrap](https://getbootstrap.com/)*
  ```bash
  npm install bootstrap@latest
  ```
* For sorting
  ```bash
  npm install @angular/animations@latest
  ```
* If you are gonna use the filtering code
  ```bash
  npm install @angular/forms@latest
  ```

### Getting started (table with filtering and sorting)
[![Demo video](https://share.dafnik.me/dfx-bootstrap-table-demo/dfx_bootstrap_table_demo.gif)](https://share.dafnik.me/dfx-bootstrap-table-demo/dfx_bootstrap_table_demo.mp4)
[Click me (or the video) for a faster version](https://share.dafnik.me/dfx-bootstrap-table-demo/dfx_bootstrap_table_demo.mp4)

You can run the demo with `npm run demo` and vist it under [`http://localhost:4200`](http://localhost:4200). [more here](#running-the-demo);

Every code piece is located in `src/app/`.

[app.component.html](./src/app/app.component.html)
```angular2html
<!-- Filtering stuff -->
<form>
  <div class='input-group'>
    <input class='form-control' type='text' [formControl]='filter'
           placeholder='Search' />
  </div>
</form>

<table ngb-table [dataSource]='dataSource' ngb-sort>
  <ng-container ngbColumnDef='id'>
    <th *ngbHeaderCellDef ngb-header-cell ngb-sort-header>#</th>
    <td *ngbCellDef='let event' ngb-cell>{{ event.id }}</td>
  </ng-container>

  <ng-container ngbColumnDef='name'>
    <th *ngbHeaderCellDef ngb-header-cell ngb-sort-header>Name</th>
    <td *ngbCellDef='let event' ngb-cell>{{ event.name }}</td>
  </ng-container>

  <ng-container ngbColumnDef='actions'>
    <th *ngbHeaderCellDef ngb-header-cell>Actions</th>
    <td *ngbCellDef='let event' ngb-cell>
      <button
        type='button'
        class='btn btn-sm m-1 btn-outline-success'>
        Edit
      </button>
      <button
        type='button'
        class='btn btn-sm m-1 btn-outline-danger'>
        Delete
      </button>
    </td>
  </ng-container>

  <tr *ngbHeaderRowDef='columnsToDisplay' ngb-header-row></tr>
  <tr *ngbRowDef='let event; columns: columnsToDisplay' ngb-row></tr>
</table>
```
[app.component.ts](./src/app/app.component.ts)
```typescript
export type eventModel = {
  id: number,
  name: string,
}

@Component({
  selector: '...'
})
export class AppComponent implements OnInit {
  // Filtering
  public filter = new FormControl();

  // Sorting
  @ViewChild(NgbSort, {static: true}) sort: NgbSort | undefined;

  public columnsToDisplay = ['id', 'name', 'actions'];
  public dataSource: NgbTableDataSource<eventModel> = new NgbTableDataSource();

  eventModels = [
    {
      id: 0,
      name: 'Event 1'
    },
    {
      id: 1,
      name: 'Event 2'
    },
    {
      id: 2,
      name: 'Event 3'
    },
    {
      id: 3,
      name: 'Event 4'
    },
  ];

  ngOnInit(): void {
    this.dataSource = new NgbTableDataSource<eventModel>(this.eventModels);
    // Sort has to be set after template initializing
    this.dataSource.sort = this.sort;

    this.filter.valueChanges.subscribe((value) => {
      this.dataSource.filter = value;
    });
  }
}
```
[app.module.ts](./src/app/app.module.ts)
```typescript
import {BrowserAnimationsModule} from '@angular/platform-browser/animations';
import {ReactiveFormsModule} from '@angular/forms';

import {DfxTableModule, DfxSortModule} from 'dfx-bootstrap-table';

@NgModule({
  declarations: [...],
  imports: [
    BrowserAnimationsModule, // (probably) only once in your project
    ReactiveFormsModule, // only if you use the filtering code
    DfxTableModule,
    DfxSortModule,
  ],
})
export class EventsModule {
}
```

## Development
Everything important in this library is located in `projects/dfx-bootstrap-table`, thats the "real" library. (following commands still have to be executed at the root level)

### Dependency installation
```bash
npm install
```

### Running the demo
[Install all dependencies](#dependency-installation). Then run:
```bash
npm run demo
```

Visit the demo under [`http://localhost:4200`](http://localhost:4200)

### Starting in development environment
```bash
npm run-script watch
```

### Building a production version
```bash
npm run-script build
```

## Deployment notes
dfx-bootstrap-table deployments are managed via [.gitlab-ci](https://gitlab.com/DatePoll/common/dfx-bootstrap-table/-/blob/develop/.gitlab-ci.yml)

All builds are uploaded to [releases.datepoll.org/common/dfx-bootstrap-table](https://releases.datepoll.org/common/dfx-bootstrap-table)

### Development builds
Commits to the [develop](https://gitlab.com/DatePoll/common/dfx-bootstrap-table/-/tree/develop) branch create a dev build downloadable via
[this link](https://releases.datepoll.org/common/dfx-bootstrap-table/dfx-bootstrap-table-dev.zip).

### Production builds
Tags create a release build downloadable via [this link](https://releases.datepoll.org/common/dfx-bootstrap-table/dfx-bootstrap-table-latest.zip). Additionally a versioned zip is uploaded and the package is published to
[npm](https://www.npmjs.com/package/dfx-bootstrap-table).
